@@ -1,15 +1,15 @@
 # -*- perl -*-
 
 #
-# $Id: XMLViewer.pm,v 1.39 2007/10/12 06:34:32 eserte Exp $
+# $Id: XMLViewer.pm,v 1.24 2001/04/29 09:10:30 eserte Exp $
 # Author: Slaven Rezic
 #
-# Copyright  2000, 2003, 2004, 2007 Slaven Rezic. All rights reserved.
+# Copyright  2000 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
-# Mail: srezic@cpan.org
-# WWW:  http://www.sourceforge.net/projects/srezic
+# Mail: mailto:eserte@cs.tu-berlin.de
+# WWW:  http://www.cs.tu-berlin.de/~eserte/
 #
 
 package Tk::XMLViewer;
@@ -20,18 +20,15 @@ require Tk::Pixmap;
 use strict;
 use vars qw($VERSION);
 
-use base qw(Tk::Derived Tk::ROText);
+use base qw(Tk::ROText);
 use XML::Parser;
 
 Construct Tk::Widget 'XMLViewer';
 
-$VERSION = '0.18_92';
+$VERSION = '0.13';
 
 my($curr_w); # ugly, but probably faster than defining handlers for everything
-my $curr_xpath;
 my $indent_width = 32;
-my $use_elide = $Tk::VERSION < 800 || $Tk::VERSION >= 804.025;
-my @tagAdds;
 
 sub SetIndent {
     my $w = shift;
@@ -39,29 +36,23 @@ sub SetIndent {
     $indent_width = $arg;
 }
 
-sub Populate {
+sub InitObject {
     my($w,$args) = @_;
-    $w->SUPER::Populate($args);
+    $w->SUPER::InitObject($args);
     $w->configure(-wrap   => 'word',
 		  -cursor => 'left_ptr');
-
-    my $tagcolor     = delete $args->{-tagcolor}     || 'red';
-    my $attrkeycolor = delete $args->{-attrkeycolor} || 'green4';
-    my $attrvalcolor = delete $args->{-attrvalcolor} || 'DarkGreen';
-    my $commentcolor = delete $args->{-commentcolor} || 'gold2';
-
     $w->tagConfigure('xml_tag',
-		     -foreground => $tagcolor,
+		     -foreground => 'red',
 		     #-font => 'boldXXX',
 		     );
     $w->tagConfigure('xml_attrkey',
-		     -foreground => $attrkeycolor,
+		     -foreground => 'green4',
 		     );
     $w->tagConfigure('xml_attrval',
-		     -foreground => $attrvalcolor,
+		     -foreground => 'DarkGreen',
 		     );
     $w->tagConfigure('xml_comment',
-		     -foreground => $commentcolor,
+		     -foreground => 'gold2',
 		     );
     $w->{IndentTags}  = [];
     $w->{RegionCount} = 0;
@@ -76,26 +67,22 @@ sub insertXML {
     my $w = shift;
     $w->Busy();
     my(%args) = @_;
-    my $xmlparserargs = delete $args{-xmlparserargs};
     my $p1 = new XML::Parser(Style => "Stream",
 			     Handlers => {
   				 Comment => \&hComment,
   				 XMLDecl => \&hDecl,
   				 Doctype => \&hDoctype,
-  			     },
-			     $xmlparserargs ? %$xmlparserargs : (),
-			    );
+  			     });
     $w->{Indent} = 0;
     $w->{PendingEnd} = 0;
     $curr_w = $w;
-    @tagAdds=();
     eval {
 	if ($args{-file}) {
-	    $w->{Source} = ['file', $args{-file}];
 	    $p1->parsefile($args{-file});
+	    $w->{Source} = ['file', $args{-file}];
 	} elsif (exists $args{-text}) {
-	    $w->{Source} = ['text', $args{-text}];
 	    $p1->parse($args{-text});
+	    $w->{Source} = ['text', $args{-text}];
 	} else {
 	    die "-text or -file argument missing";
 	}
@@ -128,10 +115,6 @@ sub insertXML {
 	}
     } else {
 	$w->_flush;
-	for (reverse @tagAdds) {
-	    $w->tagAdd(@$_);
-	}
-	@tagAdds = ();
     }
     $w->Unbusy();
 }
@@ -174,16 +157,10 @@ sub _flush {
     }
 }
 
-sub StartDocument {
-    $curr_xpath = "";
-}
-
 sub StartTag {
     $curr_w->_flush;
     my $start = $curr_w->index("end - 1 chars");
-    my $tagname = $_[1];
-    $curr_xpath .= "/" . $tagname;
-    $curr_w->insert("end", "<" . _convert_from_unicode($tagname), ['xml_tag', 'xpath_' . $curr_xpath]);
+    $curr_w->insert("end", "<" . _convert_from_unicode($_[1]), 'xml_tag');
     if (%_) {
 	$curr_w->insert("end", " ");
 	my $need_space = 0;
@@ -194,7 +171,7 @@ sub StartTag {
 		$need_space++;
 	    }
 	    $curr_w->insert("end",
-			    _convert_from_unicode($k), ["xml_attrkey", 'xpath_' . $curr_xpath . '/@' . $k],
+			    _convert_from_unicode($k), "xml_attrkey",
 			    "=\"", "",
 			    _convert_from_unicode($v), "xml_attrval",
 			    "\"", "");
@@ -258,21 +235,15 @@ sub EndTag {
 	my $start = $curr_w->index("end - 1 chars");
 	$curr_w->insert("end", "</" . _convert_from_unicode($_[1]) .">",
 			'xml_tag');
-# 	$curr_w->tagAdd($curr_w->_indenttag, $start, "end");
-	my $end_index = $curr_w->index("end");
-	push @tagAdds, [$curr_w->_indenttag, $start, $end_index];
- 	my $region_count = $curr_w->{RegionCount};
-# 	$curr_w->tagAdd("region" . $region_count,
-# 			$region_start, $region_end);
-	push @tagAdds, ["region" . $region_count,
- 			$region_start, $region_end];
+	$curr_w->tagAdd($curr_w->_indenttag, $start, "end");
+	my $region_count = $curr_w->{RegionCount};
+	$curr_w->tagAdd("region" . $region_count,
+			$region_start, $region_end);
 
-  	$curr_w->imageCreate("$tag_start",
-  			     -image => $curr_w->{'MinusImage'});
-# 	$curr_w->tagAdd("plus" . $region_count,	$tag_start);
-	push @tagAdds, ["plus" . $region_count,	$tag_start];
-# 	$curr_w->tagAdd($curr_w->_indenttag,	$tag_start);
-	push @tagAdds, [$curr_w->_indenttag,	$tag_start];
+ 	$curr_w->imageCreate("$tag_start",
+ 			     -image => $curr_w->{'MinusImage'});
+	$curr_w->tagAdd("plus" . $region_count,	$tag_start);
+	$curr_w->tagAdd($curr_w->_indenttag,	$tag_start);
 	my $ww = $curr_w;
  	$curr_w->tagBind("plus" . $region_count,
  			 '<1>' => [$ww, 'ShowHideRegion', $region_count]);
@@ -283,8 +254,6 @@ sub EndTag {
 	$curr_w->{RegionCount}++;
 	$curr_w->insert("end", "\n");
     }
-
-    $curr_xpath =~ s{/[^/]+$}{};
 }
 
 sub ShowHideRegion {
@@ -297,24 +266,16 @@ sub ShowHideRegion {
     my(@old_tags) = $w->tagNames("showhidemarkbegin");
     $w->delete("showhidemarkbegin", "showhidemarkend");
     if (!exists $args{-open}) {
-	if ($use_elide) {
-	    $args{-open} = $w->tagCget("region" . $region, '-elide');
-	} else {
-	    $args{-open} = $w->tagCget("region" . $region, '-state') eq 'hidden';
-	}
+	$args{-open} = $w->tagCget("region" . $region, -elide);
     }
     if ($args{-open}) {
 	$w->imageCreate("showhidemarkbegin",
 			-image => $w->{'MinusImage'});
-	$w->tagConfigure("region" . $region,
-			 $use_elide ? (-elide => undef)
-			            : (-state => ''));
+	$w->tagConfigure("region" . $region, -elide => undef);
     } else {
 	$w->imageCreate("showhidemarkbegin",
 			-image => $w->{'PlusImage'});
-	$w->tagConfigure("region" . $region,
-			 $use_elide ? (-elide => 1)
-			            : (-state => 'hidden'));
+	$w->tagConfigure("region" . $region, -elide => 1);
     }
     # restore old tags for minus/plus image
     foreach my $tag (@old_tags) {
@@ -438,7 +399,7 @@ warn $tag;
 
 sub XMLMenu {
     my $w = shift;
-    if ($Tk::VERSION > 800.014 && $w->can("menu")) {
+    if ($w->can("menu")) {
 	my $textmenu = $w->menu;
 	my $xmlmenu = $textmenu->cascade(-tearoff => 0,
 					 -label => "XML");
@@ -453,63 +414,13 @@ sub XMLMenu {
 	}
 	$depthmenu->command(-label => "Open all",
 			    -command => sub { $w->ShowToDepth(undef) });
-	$xmlmenu->command(-label => "XPath to Selection",
-			  -command => sub { $w->XPathToSelection });
 # XXX not yet:
 #	$xmlmenu->command(-label => "Close selected region",
 #			  -command => sub { $w->CloseSelectedRegion });
     }
 }
 
-sub XPathToSelection {
-    my($w) = @_;
-    my($X,$Y) = @{$w->{PostPosition}};
-    my $xpath = $w->GetXPathFromXY($X, $Y);
-
-    # Define a dummy widget holding the selection, so we can still use
-    # the Text selection after using XPathToSelection
-    my $dummy = $w->Subwidget("DummyLabelForSelection");
-    if (!$dummy) {
-	$dummy = $w->Component("Label" => "DummyLabelForSelection");
-    }
-
-    $dummy->SelectionOwn;
-    $dummy->SelectionHandle
-	(sub {
-	     my($offset, $maxbytes) = @_;
-	     substr($xpath, $offset, $maxbytes);
-	 });
-}
-
-sub PostPopupMenu {
-    my $w = shift;
-    my $X = shift;
-    my $Y = shift;
-    $w->{PostPosition} = [$X,$Y];
-    $w->SUPER::PostPopupMenu($X, $Y, @_);
-}
-
-sub BalloonInfo {
-    my($w,$balloon,$X,$Y,@opt) = @_;
-    $w->GetXPathFromXY($X, $Y);
-}
-
-sub GetXPathFromXY {
-    my($w, $X, $Y) = @_;
-    for my $tag ($w->tagNames('@'.($X-$w->rootx).','.($Y-$w->rooty))) {
-	if ($tag =~ m{^xpath_(.*)$}) {
-	    my $xpath = $1;
-	    return $xpath;
-	}
-    }
-    return '';
-}
-
-if ($Tk::VERSION >= 803) { # native unicode support
-    eval <<'EOF';
-sub _convert_from_unicode { $_[0] }
-EOF
-} elsif ($] >= 5.006001) {
+if ($] >= 5.006001) {
     # tr translator for unicode not available anymore
     eval <<'EOF';
 sub _convert_from_unicode {
@@ -537,8 +448,8 @@ sub _convert_from_unicode {
 }
 EOF
     } else { # do nothing
-        warn "No unicode decoder found --- consider installing Unicode::String";
         eval <<'EOF';
+require Unicode::String;
 sub _convert_from_unicode { $_[0] }
 EOF
     }
@@ -607,66 +518,22 @@ Tk::XMLViewer is an widget inherited from Tk::Text which displays XML
 in a hierarchical tree. You can use the plus and minus buttons to
 hide/show parts of the tree.
 
-=head2 OPTIONS
-
-C<Tk::XMLViewer> supports all option of C<Tk::Text> and additionally
-the following:
-
-=over
-
-=item -tagcolor => $color
-
-Foreground color of tags.
-
-=item -attrkeycolor => $color
-
-Foreground color of attribute keys.
-
-=item -attrvalcolor => $color
-
-Foreground color of attribute values.
-
-=item -commentcolor => $color
-
-Foreground color of comment sections.
-
-=back
-
-The text tags C<xml_tag>, C<xml_attrkey>, C<xml_attrval>, and
-C<xml_comment> are defined for the corresponding XML elements. If you
-want to customize further you can configure the tags directly, for
-example:
-
-    $xmlviewer->tagConfigure('xml_comment', -foreground => "white",
-			     -background => "red", -font => "Helvetica 6");
-
-
-=head2 METHODS
+=head1 METHODS
 
 =over 4
 
 =item insertXML
 
-Insert XML into the XMLViewer widget. Use the B<-file> argument to
-insert a file and B<-text> to insert an XML string. A hash to the
-B<-xmlparserargs> option will be passed to the L<XML::Parser>
-constructor.
+Insert XML into the XMLViewer widget. Use the -file argument to insert
+a file and -text to insert an XML string.
 
 =item DumpXML
 
-Dump the contents of an C<Tk::Text> widget into an XML string. This is
-meant as a alternative to the C<Tk::Text::dump> method (in fact,
-C<DumpXML> is implemented with the help of C<dump>).
-
-The output of C<DumpXML> can be used as input for the XMLViewer
-widget, which is useful in debugging C<Tk::Text> tags.
-
-Use the static variant of C<DumpXML> for C<Tk::Text> widgets and the
-method variant for C<XMLViewer> widgets.
+Dump the contents of an Tk::Text widget into an XML string, which can
+be used as input for the XMLViewer widget. Use the static variant for
+Tk::Text widgets and the method for XMLViewer widgets.
 
     $xml_string1 = Tk::XMLViewer::DumpXML($text_widget);
-    $xmlviewer_widget->insertXML($xml_string1);
-
     $xml_string2 = $xmlviewer->DumpXML;
 
 =item SetIndent
@@ -701,21 +568,13 @@ Elements for DOCTYPE: Name Sysid Pubid Internal
 
 =back
 
-=head1 NOTES
-
-=head2 Unicode
-
-Perl/Tk 800 does not support Unicode. In this case C<Tk::XMLViewer>
-tries to translate all characters returned by the XML parser to the
-C<iso-8859-1> charset. This may be done with a builtin function like
-C<pack>/C<unpack> or a CPAN module like L<Unicode::String>. If no
-fallback could be found, then unicode characters show as binary
-values.
-
-Perl/Tk 804 has Unicode support.
-
 =head1 BUGS
 
+Perl/Tk does not support Unicode, yet. For perl 5.6.0 and newer,
+unicode characters are translated to ISO-8859-1 chars, if possible.
+For older perls, there is no conversion (unless Unicode::String is
+installed), so unicode characters will show as binary values.
+
 DumpXML will not work with nested text tags.
 
 There should be only one insertXML operation at one time (these is
@@ -732,12 +591,12 @@ head1 TODO
 
 =head1 AUTHOR
 
-Slaven Rezic, <slaven@rezic.de>
+Slaven Rezic, <eserte@cs.tu-berlin.de>
 
 Some additions by Jerry Geiger <jgeiger@rios.de>.
 
 =head1 SEE ALSO
 
-L<XML::Parser>, L<Tk::Text>, L<tkxmlview>.
+L<XML::Parser>(3), L<Tk::Text>(3).
 
 =cut
